@@ -241,10 +241,14 @@ ply_terminal_set_unbuffered_input (ply_terminal_t *terminal)
 
         ply_terminal_unlock (terminal);
 
-        terminal->is_disabled = false;
+        if (terminal->is_disabled) {
+                ply_trace ("terminal input is getting enabled in unbuffered mode");
 
-        if (ply_terminal_is_vt (terminal))
-                ioctl (terminal->fd, KDSKBMODE, K_UNICODE);
+                if (ply_terminal_is_vt (terminal))
+                        ioctl (terminal->fd, KDSKBMODE, K_UNICODE);
+
+                terminal->is_disabled = false;
+        }
 
         tcgetattr (terminal->fd, &term_attributes);
 
@@ -276,10 +280,14 @@ ply_terminal_set_buffered_input (ply_terminal_t *terminal)
 {
         struct termios term_attributes;
 
-        terminal->is_disabled = false;
+        if (terminal->is_disabled) {
+                ply_trace ("terminal input is getting enabled in buffered mode");
 
-        if (ply_terminal_is_vt (terminal))
-                ioctl (terminal->fd, KDSKBMODE, K_UNICODE);
+                if (ply_terminal_is_vt (terminal))
+                        ioctl (terminal->fd, KDSKBMODE, K_UNICODE);
+
+                terminal->is_disabled = false;
+        }
 
         if (!terminal->is_unbuffered)
                 return true;
@@ -324,10 +332,15 @@ ply_terminal_set_buffered_input (ply_terminal_t *terminal)
 bool
 ply_terminal_set_disabled_input (ply_terminal_t *terminal)
 {
-        terminal->is_disabled = true;
+        if (!terminal->is_disabled) {
+                ply_trace ("terminal input is getting disabled from %s mode",
+                           terminal->is_unbuffered? "unbuffered" : "buffered");
 
-        if (ply_terminal_is_vt (terminal))
-                ioctl (terminal->fd, KDSKBMODE, K_OFF);
+                if (ply_terminal_is_vt (terminal))
+                        ioctl (terminal->fd, KDSKBMODE, K_OFF);
+
+                terminal->is_disabled = true;
+        }
 
         return true;
 }
