@@ -19,7 +19,6 @@
  *
  * Written by: Ray Strode <rstrode@redhat.com>
  */
-#include "config.h"
 
 #include <assert.h>
 #include <dirent.h>
@@ -78,7 +77,8 @@ struct _ply_throbber
         uint32_t             is_stopped : 1;
 };
 
-static void ply_throbber_stop_now (ply_throbber_t *throbber);
+static void ply_throbber_stop_now (ply_throbber_t *throbber,
+                                   bool            redraw);
 
 ply_throbber_t *
 ply_throbber_new (const char *image_dir,
@@ -126,7 +126,7 @@ ply_throbber_free (ply_throbber_t *throbber)
                 return;
 
         if (!throbber->is_stopped)
-                ply_throbber_stop_now (throbber);
+                ply_throbber_stop_now (throbber, false);
 
         ply_throbber_remove_frames (throbber);
         ply_array_free (throbber->frames);
@@ -161,7 +161,7 @@ animate_at_time (ply_throbber_t *throbber,
                  * frame and loop around. Clamp it to the last frame.
                  */
                 if (last_frame_number > throbber->frame_number)
-                      throbber->frame_number = number_of_frames - 1;
+                        throbber->frame_number = number_of_frames - 1;
 
                 if (throbber->frame_number == number_of_frames - 1)
                         should_continue = false;
@@ -281,7 +281,7 @@ out:
         }
         free (entries);
 
-        return (ply_array_get_size (throbber->frames) > 0);
+        return ply_array_get_size (throbber->frames) > 0;
 }
 
 bool
@@ -304,7 +304,9 @@ ply_throbber_start (ply_throbber_t      *throbber,
                     long                 y)
 {
         assert (throbber != NULL);
-        assert (throbber->loop == NULL);
+
+        if (!throbber->is_stopped)
+                ply_throbber_stop_now (throbber, false);
 
         throbber->loop = loop;
         throbber->display = display;
@@ -324,15 +326,19 @@ ply_throbber_start (ply_throbber_t      *throbber,
 }
 
 static void
-ply_throbber_stop_now (ply_throbber_t *throbber)
+ply_throbber_stop_now (ply_throbber_t *throbber,
+                       bool            redraw)
 {
         throbber->is_stopped = true;
 
-        ply_pixel_display_draw_area (throbber->display,
-                                     throbber->x,
-                                     throbber->y,
-                                     throbber->frame_area.width,
-                                     throbber->frame_area.height);
+        if (redraw) {
+                ply_pixel_display_draw_area (throbber->display,
+                                             throbber->x,
+                                             throbber->y,
+                                             throbber->frame_area.width,
+                                             throbber->frame_area.height);
+        }
+
         if (throbber->loop != NULL) {
                 ply_event_loop_stop_watching_for_timeout (throbber->loop,
                                                           (ply_event_loop_timeout_handler_t)
@@ -356,7 +362,7 @@ ply_throbber_stop (ply_throbber_t *throbber,
         }
 
         if (stop_trigger == NULL) {
-                ply_throbber_stop_now (throbber);
+                ply_throbber_stop_now (throbber, true);
                 return;
         }
 
@@ -401,4 +407,3 @@ ply_throbber_get_height (ply_throbber_t *throbber)
         return throbber->height;
 }
 
-/* vim: set ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
