@@ -19,7 +19,6 @@
  *
  * Written by: Charlie Brej <cbrej@cs.man.ac.uk>
  */
-#include "config.h"
 #include "ply-hashtable.h"
 #include "ply-utils.h"
 #include "ply-bitarray.h"
@@ -275,4 +274,3 @@ ply_hashtable_get_size (ply_hashtable_t *hashtable)
         return hashtable->live_node_count;
 }
 
-/* vim: set ts=4 sw=4 expandtab autoindent cindent cino={.5s,(0: */
