@@ -19,7 +19,6 @@
  *
  * Written by: Charlie Brej <cbrej@cs.man.ac.uk>
  */
-#include "config.h"
 #include "ply-image.h"
 #include "ply-utils.h"
 #include "ply-logger.h"
@@ -223,17 +222,7 @@ static script_return_t sprite_window_get_width (script_state_t *state,
                 return script_return_obj (script_obj_new_number (width));
         }
 
-        width = 0;
-        for (node = ply_list_get_first_node (data->displays);
-             node;
-             node = ply_list_get_next_node (data->displays, node)) {
-                display = ply_list_node_get_data (node);
-                if (width == 0)
-                        width = ply_pixel_display_get_width (display->pixel_display);
-                else
-                        width = MIN (width, ply_pixel_display_get_width (display->pixel_display));
-        }
-        return script_return_obj (script_obj_new_number (width));
+        return script_return_obj (script_obj_new_number (data->max_width));
 }
 
 static script_return_t sprite_window_get_height (script_state_t *state,
@@ -261,17 +250,7 @@ static script_return_t sprite_window_get_height (script_state_t *state,
                 return script_return_obj (script_obj_new_number (height));
         }
 
-        height = 0;
-        for (node = ply_list_get_first_node (data->displays);
-             node;
-             node = ply_list_get_next_node (data->displays, node)) {
-                display = ply_list_node_get_data (node);
-                if (height == 0)
-                        height = ply_pixel_display_get_height (display->pixel_display);
-                else
-                        height = MIN (height, ply_pixel_display_get_height (display->pixel_display));
-        }
-        return script_return_obj (script_obj_new_number (height));
+        return script_return_obj (script_obj_new_number (data->max_height));
 }
 
 static script_return_t sprite_window_get_x (script_state_t *state,
@@ -416,8 +395,8 @@ static script_return_t sprite_window_set_background_bottom_color (script_state_t
         return script_return_obj_null ();
 }
 
-static void script_lib_draw_brackground (ply_pixel_buffer_t *pixel_buffer,
-                                         ply_rectangle_t *clip_area,
+static void script_lib_draw_brackground (ply_pixel_buffer_t       *pixel_buffer,
+                                         ply_rectangle_t          *clip_area,
                                          script_lib_sprite_data_t *data)
 {
         if (data->background_color_start == data->background_color_end) {
@@ -451,19 +430,22 @@ static void script_lib_sprite_draw_area (script_lib_display_t *display,
 
 
         node = ply_list_get_first_node (data->sprite_list);
+        if (node == NULL)
+                return;
+
         sprite = ply_list_node_get_data (node);
 
         /* Check If the first sprite should be rendered opaque */
         if (sprite->image && !sprite->remove_me &&
-            ply_pixel_buffer_is_opaque (sprite->image) && sprite->opacity == 1.0)  {
+            ply_pixel_buffer_is_opaque (sprite->image) && sprite->opacity == 1.0) {
                 int position_x = sprite->x - display->x;
                 int position_y = sprite->y - display->y;
 
                 /* In that case only draw the background if the sprite doesn't
                  * cover the complete area */
                 if (position_x > x || position_y > y ||
-                    ((int)ply_pixel_buffer_get_width (sprite->image) + position_x) < (x + width) ||
-                    ((int)ply_pixel_buffer_get_height (sprite->image) + position_y) < (y + height))
+                    ((int) ply_pixel_buffer_get_width (sprite->image) + position_x) < (x + width) ||
+                    ((int) ply_pixel_buffer_get_height (sprite->image) + position_y) < (y + height))
                         script_lib_draw_brackground (pixel_buffer, &clip_area, data);
         } else {
                 script_lib_draw_brackground (pixel_buffer, &clip_area, data);
@@ -519,47 +501,68 @@ draw_area (script_lib_sprite_data_t *data,
         }
 }
 
+static void
+update_displays (script_lib_sprite_data_t *data)
+{
+        ply_list_node_t *node;
+        script_lib_display_t *script_display;
+
+        data->max_width = 0;
+        data->max_height = 0;
+        for (node = ply_list_get_first_node (data->displays);
+             node;
+             node = ply_list_get_next_node (data->displays, node)) {
+                script_display = ply_list_node_get_data (node);
+                data->max_width = MAX (data->max_width, ply_pixel_display_get_width (script_display->pixel_display));
+                data->max_height = MAX (data->max_height, ply_pixel_display_get_height (script_display->pixel_display));
+        }
+
+        for (node = ply_list_get_first_node (data->displays);
+             node;
+             node = ply_list_get_next_node (data->displays, node)) {
+                script_display = ply_list_node_get_data (node);
+                script_display->x = (data->max_width - ply_pixel_display_get_width (script_display->pixel_display)) / 2;
+                script_display->y = (data->max_height - ply_pixel_display_get_height (script_display->pixel_display)) / 2;
+        }
+
+        data->full_refresh = true;
+}
+
+static void
+add_display (script_lib_sprite_data_t *data,
+             ply_pixel_display_t      *pixel_display)
+{
+        script_lib_display_t *script_display = malloc (sizeof(script_lib_display_t));
+
+        script_display->pixel_display = pixel_display;
+        script_display->data = data;
+        ply_pixel_display_set_draw_handler (pixel_display,
+                                            (ply_pixel_display_draw_handler_t)
+                                            script_lib_sprite_draw_area, script_display);
+
+        ply_list_append_data (data->displays, script_display);
+}
+
 script_lib_sprite_data_t *script_lib_sprite_setup (script_state_t *state,
                                                    ply_list_t     *pixel_displays)
 {
         ply_list_node_t *node;
-        unsigned int max_width, max_height;
         script_lib_sprite_data_t *data = malloc (sizeof(script_lib_sprite_data_t));
 
         data->class = script_obj_native_class_new (sprite_free, "sprite", data);
         data->sprite_list = ply_list_new ();
         data->displays = ply_list_new ();
 
-        max_width = 0;
-        max_height = 0;
-
         for (node = ply_list_get_first_node (pixel_displays);
              node;
              node = ply_list_get_next_node (pixel_displays, node)) {
                 ply_pixel_display_t *pixel_display = ply_list_node_get_data (node);
-                max_width = MAX (max_width, ply_pixel_display_get_width (pixel_display));
-                max_height = MAX (max_height, ply_pixel_display_get_height (pixel_display));
-        }
-
-        for (node = ply_list_get_first_node (pixel_displays);
-             node;
-             node = ply_list_get_next_node (pixel_displays, node)) {
-                ply_pixel_display_t *pixel_display = ply_list_node_get_data (node);
-                script_lib_display_t *script_display = malloc (sizeof(script_lib_display_t));
-                script_display->pixel_display = pixel_display;
-
-                script_display->x = (max_width - ply_pixel_display_get_width (pixel_display)) / 2;
-                script_display->y = (max_height - ply_pixel_display_get_height (pixel_display)) / 2;
-
-                script_display->data = data;
-                ply_pixel_display_set_draw_handler (pixel_display,
-                                                    (ply_pixel_display_draw_handler_t)
-                                                    script_lib_sprite_draw_area, script_display);
-
-                ply_list_append_data (data->displays, script_display);
+                add_display (data, pixel_display);
         }
+        update_displays (data);
 
         script_obj_t *sprite_hash = script_obj_hash_get_element (state->global, "Sprite");
+
         script_add_native_function (sprite_hash,
                                     "_New",
                                     sprite_new,
@@ -624,6 +627,7 @@ script_lib_sprite_data_t *script_lib_sprite_setup (script_state_t *state,
 
 
         script_obj_t *window_hash = script_obj_hash_get_element (state->global, "Window");
+
         script_add_native_function (window_hash,
                                     "GetWidth",
                                     sprite_window_get_width,
@@ -685,12 +689,14 @@ script_lib_sprite_data_t *script_lib_sprite_setup (script_state_t *state,
         data->background_color_end = 0x000000;
         data->full_refresh = true;
         script_return_t ret = script_execute (state, data->script_main_op);
+
         script_obj_unref (ret.object);
         return data;
 }
 
 static int
-sprite_compare_z (void *data_a, void *data_b)
+sprite_compare_z (void *data_a,
+                  void *data_b)
 {
         sprite_t *sprite_a = data_a;
         sprite_t *sprite_b = data_b;
@@ -714,27 +720,38 @@ region_add_area (ply_region_t *region,
         ply_region_add_rectangle (region, &rectangle);
 }
 
-void script_lib_sprite_pixel_display_removed (script_lib_sprite_data_t *data, ply_pixel_display_t *pixel_display)
+void script_lib_sprite_pixel_display_added (script_lib_sprite_data_t *data,
+                                            ply_pixel_display_t      *pixel_display)
 {
-    ply_list_node_t *node;
-    ply_list_node_t *next_node;
-    script_lib_display_t* display;
+        add_display (data, pixel_display);
+        update_displays (data);
+}
 
-    if (!data)
-        return;
+void script_lib_sprite_pixel_display_removed (script_lib_sprite_data_t *data,
+                                              ply_pixel_display_t      *pixel_display)
+{
+        ply_list_node_t *node;
+        ply_list_node_t *next_node;
+        script_lib_display_t *display;
+        bool update = false;
 
-    node = ply_list_get_first_node (data->displays);
-    while (node)
-    {
-        next_node = ply_list_get_next_node (data->displays, node);
-        display = ply_list_node_get_data (node);
+        if (!data)
+                return;
 
-        if (display->pixel_display == pixel_display)
-        {
-            ply_list_remove_node (data->displays, node);
+        node = ply_list_get_first_node (data->displays);
+        while (node) {
+                next_node = ply_list_get_next_node (data->displays, node);
+                display = ply_list_node_get_data (node);
+
+                if (display->pixel_display == pixel_display) {
+                        ply_list_remove_node (data->displays, node);
+                        update = true;
+                }
+                node = next_node;
         }
-        node = next_node;
-    }
+
+        if (update)
+                update_displays (data);
 }
 
 void
@@ -745,15 +762,12 @@ script_lib_sprite_refresh (script_lib_sprite_data_t *data)
         ply_list_t *rectable_list;
 
         if (!data)
-            return;
+                return;
 
         region = ply_region_new ();
 
         ply_list_sort_stable (data->sprite_list, &sprite_compare_z);
 
-        node = ply_list_get_first_node (data->sprite_list);
-
-
         if (data->full_refresh) {
                 for (node = ply_list_get_first_node (data->displays);
                      node;
@@ -769,6 +783,7 @@ script_lib_sprite_refresh (script_lib_sprite_data_t *data)
                 data->full_refresh = false;
         }
 
+        node = ply_list_get_first_node (data->sprite_list);
         while (node) {
                 sprite_t *sprite = ply_list_node_get_data (node);
                 ply_list_node_t *next_node = ply_list_get_next_node (data->sprite_list,
